﻿#!/usr/bin/python 
#coding:utf-8
#------------------------------------------------------------------------------
# Упрощенный расчет температуры кипения смеси этанол-вода в зависимости от
# мольной концентрации и давления пара для задач управления ректификационной
# установкой. Работоспособность скрипта проверена на микрокомпьютере Raspberry Pi
# со штатной установкой Raspbian (Linux 3.6.11+ armv6l) и Python 2.7.3
# 16.12.15, OldBean
#------------------------------------------------------------------------------
import sys, math
#import numpy as np
#import scipy.optimize as scio
#------------------------------------------------------------------------------
# Для расчета давления насыщенного пара чистого компонента используется уравнение
# Антуан (Antoine equation)
# Температура - °C, давление - мм.рт.ст.
# Источники, где есть коэффициенты для этанола (1) и воды (2)
# 1. https://en.wikipedia.org/wiki/Antoine_equation
#A1 = 8.20417; B1 = 1642.89; C1 = 230.300 # Диапазон -57...80°C
#A1 = 7.68117; B1 = 1332.04; C1 = 199.200 # Диапазон  77...243°C
A2 = 8.07131; B2 = 1730.63; C2 = 233.426 # Диапазон	1...100°C
#A2 = 8.14019; B2 = 1810.94; C2 = 244.485 # Диапазон	99...374°C
# 2. Yaws and Yang (Yaws, C.  L.  and Yang, H.  C., "To estimate vapor pressure
# easily. antoine coefficients relate vapor pressure to temperature for almost
# 700 major organic compounds", Hydrocarbon Processing, 68(10), p65-68, 1989.
# http://eng.umd.edu/~nsw/chbe250/antoine.dat
A1 = 8.21330; B1 = 1652.050; C1 = 231.480 # Диапазон -3...96°C
# 3. From Elliot and Lira, Introductory Chemical Engineering Thermodynamics
# http://www.personal.utulsa.edu/~geoffrey-price/Courses/ChE7023/AntoineConstants.xls
#A1 = 8.1122; B1 = 1592.864; C1 = 226.184 # Диапазон 20...93°C
# Давление насыщенного пара чистых компонентов
def Psat1(T): # Этанол
  return 10**(A1 - B1/(T + C1))
def Psat2(T): # Вода
  return 10**(A2 - B2/(T + C2))
#------------------------------------------------------------------------------
# Про интерполяционные уравнения Ван-Лаара для коэффициентов активности
# см., например, Стабников и др. "Этиловый спирт", стр.135
# Источники, в которых можно найти коэффициенты Ван-Лаара:
# 1. Демка про CDF: 
# http://demonstrations.wolfram.com/VaporLiquidEquilibriumForAnEthanolWaterMixture/
#A12 = 1.66480; A21 = 0.94010
# 2. J. Gmehling, U. Ouken, Vapor-Liquid Equilibrium Data Collection, 
# DECHEMA Chemistry Data, v.1 (parts 1-10), Frankfurt, 1977
# Для системы вода-этанол коэффициенты из этого источника приведены в статье:
# M. Hristova, D. Damgaliev, D. Popova. Estimation of Water-Alcohol Mixture Flash
# Point. Journal of the University of Chemical Technology and Metallurgy, 45, 1, 
# 2010, 19-24. http://dl.uctm.edu/journal/node/j2010-1/2_Mariana_19-24.pdf
A12 = 1.6798; A21 = 0.9227
# Коэффициенты активности
def Gam1(xa): # Этанол
  return math.exp(A12*(A21*(1.0 - xa)/(A12*xa + A21*(1.0 - xa)))**2) 
def Gam2(xa): # Вода
  return math.exp(A21*(A12*xa/(A21*(1.0 - xa) + A12*xa))**2)
#------------------------------------------------------------------------------
def Func(T): # Модифицированный закон Рауля (с активностями)
  # Корень этой функции - это и есть температура кипения
  return Psat1(Tem)*T*Gam1(T) + Psat2(Tem)*(1.0 - T)*Gam2(T) - P
#------------------------------------------------------------------------------
# Увы, нормально установить scipy на Raspberry Pi почему-то не удалось. Поэтому
# для нахождения корня здесь используется простая "самодельная" функция.
# Метод нахождения корня - деление отрезка пополам (ака метод дихотомии)
def my_root(f, aa, bb, eps = 1E-7, its = 30):
  if aa < bb:
    a = aa; b = bb
  else:
    a = bb; b = aa
  fa = f(a); fb = f(b)
  if fa*fb > 0:
    print ("В заданном интервале нет корней (или больше одного корня)")
    return 0
  it = 0; dx = b - a
  while it < its and dx > eps:
    c = 0.5*(a + b); fc = f(c)
    print (it, a, b, c, dx)
    if fa*fc >= 0: 
      a = c; fa = fc
    else: 
      b = c; fb = fc
    dx = b - a; it += 1
  return 0.5*(a + b)
#------------------------------------------------------------------------------
if __name__ == "__main__":
  if len (sys.argv) > 1: P = float(sys.argv[1]) 
  else: P = 760
#  print ("P = ", P, " mmHg")
#  print ("x, mole/mole\tT°C\ty, mole/mole")

  P = 750
  Tem = 85
  Mol = my_root(Func, 0, 1)
  print (Tem, "\t", P, "\t", Mol*100)
 #   ya = Psat1(Tb)*xa*Gam1(xa)/P
 #   print (xa, "\t", Tb, "\t", ya)
